import { DissonatorDetails, GlobalConfig, BotConfig } from '../types';
import { TextChannel, Message } from 'discord.js';
import log from 'npmlog';
import {
  GoogleSpreadsheet,
  GoogleSpreadsheetWorksheet,
} from 'google-spreadsheet';
import { Bot, sendTimedRandomMessages, asyncFilter, random } from '../common';

export class Dissonator extends Bot<DissonatorDetails> {
  phrasesSheet: GoogleSpreadsheetWorksheet;
  constructor(
    globalConfig: GlobalConfig,
    botConfig: BotConfig<DissonatorDetails>,
    spreadsheet: GoogleSpreadsheet,
    phrasesSheetId: string
  ) {
    super(botConfig, globalConfig.generalChannelId);
    this.phrasesSheet = spreadsheet.sheetsById[phrasesSheetId];

    //  this.discordClient.on('message', async message => {
    //    useManualMessageSending.bind(this)(message);
    //  });

    this.init();
  }

  async init(): Promise<void> {
    await this.discordClient.login(this.config.token);
    sendTimedRandomMessages.bind(this)([10, 90], true);
  }
}

export class DissonatorController {
  bots: Dissonator[];
  constructor(
    bots: Dissonator[],
    controlChannelId: string,
    messageChannelId: string
  ) {
    log.info('Dissonator Controller', 'initializing dissonator controller');
    this.bots = bots;

    const messagesBeingHandled = new Set<string>();

    bots.forEach(bot => {
      bot.discordClient.on('message', (message: Message) => {
        if (messagesBeingHandled.has(message.id)) return;
        if (message.channel.id !== controlChannelId) return;
        if (message.author.bot) return;
        messagesBeingHandled.add(message.id);
        log.info(
          'Dissonator Controller',
          'got message from %s in control channel',
          message.author.username
        );
        (async () => {
          // const botClients = this.bots.map(bot => bot.discordClient);
          const allowedBots = await asyncFilter(this.bots, async bot => {
            try {
              const channel = await bot.discordClient.channels.fetch(
                messageChannelId
              );
              if (
                channel &&
                channel instanceof TextChannel &&
                channel.viewable
              ) {
                return true;
              } else {
                return false;
              }
            } catch (e) {
              log.warn(
                bot.config.name,
                'Bot is no longer able to access channel'
              );
              return false;
            }
          });
          const bot = allowedBots[random(allowedBots.length)];
          const channel = await bot.discordClient.channels.fetch(
            messageChannelId
          );
          if (channel instanceof TextChannel) {
            try {
              await channel.send(message.content);
              await message.react('✨');

              log.verbose(
                '[Dissonator Controller]' + bot.config.name,
                'message successfully sent'
              );
              messagesBeingHandled.delete(message.id);
            } catch (e) {
              log.error(
                '[Dissonator Controller]' + bot.config.name,
                'error sending or reacting to message',
                e
              );
            }
          } else {
            log.error(
              '[Dissonator Controller]' + bot.config.name,
              'could not send message because target channel is not a text channel'
            );
          }
        })();
      });
    });

    log.info('Dissonator Controller', 'controller ready');
  }
}
