import { StemswardenDetails, GlobalConfig } from '../types';
import {
  GoogleSpreadsheet,
  GoogleSpreadsheetWorksheet,
} from 'google-spreadsheet';
import {
  Bot,
  useManualMessageSending,
  sendTimedRandomMessages,
  useClueChecking,
} from '../common';

export class Stemswarden extends Bot<StemswardenDetails> {
  phrasesSheet: GoogleSpreadsheetWorksheet;
  stemsSheet: GoogleSpreadsheetWorksheet;
  constructor(config: GlobalConfig, spreadsheet: GoogleSpreadsheet) {
    const botConfig = config.bots.stemswarden;
    super(botConfig, config.generalChannelId);
    this.phrasesSheet =
      spreadsheet.sheetsById[this.config.details.sheetIds.phrases];
    this.stemsSheet =
      spreadsheet.sheetsById[this.config.details.sheetIds.stems];

    this.discordClient.on('message', async message => {
      useManualMessageSending.bind(this)(message);
      useClueChecking.bind(this)(message);
    });

    this.init();
  }

  async init(): Promise<void> {
    await this.discordClient.login(this.config.token);
    sendTimedRandomMessages.bind(this)([80, 300]);
  }
}
